function [x,y,maxL]=gm_min_praxis_L2norm(x0,y0,w,epsi,ipb,iprint);
%GM_MIN_PRAXIS_L2NORM looks for the min of the 2-norm of the Lagrange polynomials using gm_praxis

% Input:
% (x0,y0) = starting points
% w = weights (sum(w) = 1)
% epsi = stopping criteria on the value of the objective function
% ipb = problem to be considered
%        ipb = 1 square
%        ipb = 2 disk
%        ipb = 3 L-shape region
%        ipb = 4 triangle (simplex)
%        ipb = 5 double bubble
%        ipb = 6 ellipse
%        ipb = 7 half-ellipse
% iprint = 1, printing
%
% Output:
% (x,y) = new set of points
% maxL = Lebesgue const

%
% Author G. Meurant
% June 2014
% Updated August 2015
%

global iprob diam
global xnode ynode weight
global wparm

if nargin <= 4
 ipb = 1;
end

if nargin <= 5
 iprint = 1;
end

iprob = ipb;
if abs(sum(w) - 1) > 1e-14
 error('gm_min_praxis_L2norm: The sum of the weights must be equal to 1')
end % if abs
I = find(w <= 0);
if ~isempty(I)
 error('gm_min_praxis_L2norm: There are negative or zero weights')
end % if isempty
wparm = w;

n = length(x0);
X0 = [x0 y0];
n2 = 2 * n;
nftot = 0;
xp = X0;

% nodes and weights for the integral

d = ceil((-3 + sqrt(1 + 8 * n)) / 2);
d2 = 2 * d;
% You may eventually remove the next 3 lines
if d > 20
 error('gm_min_praxis_L2norm: The degree is too large')
end
d2 = min(d2,40);

switch iprob
 
 case 1
  % unit square
  diam = 2;
  % use Padua points (param = 2)
  % Int is not used
%   [Int,nodes,weight] = gm_quadPD(@gm_Lag_L2norm_squared,d2,[-1 1],[-1 1],2,X0');
  [Int,nodes,weight] = gm_quadPD(@const,d2,[-1 1],[-1 1],2,X0');
  xnode = nodes(:,1); 
  ynode = nodes(:,2);
  
 case 2
  % unit disk
  diam = 2;
  xyw = gm_gqcircsect(d2,pi,0.,1);
  xnode = xyw(:,1);
  ynode = xyw(:,2);
  weight = xyw(:,3);
  
 case 3
  % L-shape region
  diam = 2;
  polygon_sides = [-1 -1; 1 -1; 1 0; 0 0; 0 1; -1 1; -1 -1];
  xyw = gm_polygauss_2013(d2,polygon_sides);
  xnode = xyw(:,1);
  ynode = xyw(:,2);
  weight = xyw(:,3);
  
 case 4
  % simplex = triangle
  diam = 1;
  polygon_sides = [0 0; 1 0; 0 1; 0 0];
  xyw = gm_polygauss_2013(d2,polygon_sides);
  xnode = xyw(:,1);
  ynode = xyw(:,2);
  weight = xyw(:,3);
  
 case 5
  % double bubble
  % center (0,0) radius 5, center (6,0) radius 3
  diam = 14;
  xyw = gm_gqdbubble(d2,0,0,5,6,0,3);
  xnode = xyw(:,1);
  ynode = xyw(:,2);
  weight = xyw(:,3);
  
 case 6
  % ellipse with semi axes 2 (x) and 1 (y)
  diam = 4;
  xyw = gm_gqcircsect(d2,pi,0.,1);
  xnode = 2 * xyw(:,1);
  ynode = xyw(:,2);
  weight = xyw(:,3);
  
 case 7
  % half-ellipse with semi axes 2 (x) and 1 (y), y >= 0
  diam = 4;
  xyw = gm_gqcircsect(d2,pi/2,0.,1);
  xnode = 2 * xyw(:,2);
  ynode = xyw(:,1);
  weight = xyw(:,3);
  
end % switch

% rough estimate to find a "good" starting point
% [xp,prax,iter,nf,exitflag] = gm_praxis(1e-1,1e-1,diam,n2,X0,@gm_Lag_L2norm);
% fprintf('iter = %d, nfunc = %d, exitflag = %d, f = %0.5f \n',iter,nf,exitflag,prax)
% nftot = nftot + nf;

% the number of function evaluations (last parameter) may be too small

[xp,prax,iter,nf,exitflag] = gm_praxis(epsi,epsi,diam,n2,xp,@gm_Lag_L2norm,10,2500);
if iprint == 1
 fprintf(' iter = %d, nfunc = %d, exitflag = %d, f = %0.5f \n',iter,nf,exitflag,prax)
end
nftot = nftot + nf;

% improve the solution by restarting (?)
kmax = 20;
prax_old = prax;

for k = 1:kmax
 if iprint == 1
  fprintf('---------iteration %d \n',k)
 end
 xp_old = xp;
 [xp,prax,iter,nf,exitflag] = gm_praxis(epsi,epsi,diam,n2,xp,@gm_Lag_L2norm,10,2000);
 nftot = nftot + nf;
 
 if prax > 10 * prax_old
  % there is something wrong
  xp = xp_old;
  prax = prax_old;
  x = xp(1:n)'; 
  y = xp(n+1:n2)';
  if iprint == 1
   fprintf(' iter = %d, nfunc = %d, exitflag = %d, f = %0.5f \n',iter,nf,exitflag,prax)
   fprintf(' total nfunc = %d \n',nftot)
  end
  % this may not be enough to compute the L-constant reliably!!!!!!
  [Psinp,maxL,Psidot,XY] = gm_comp_Lebesgue_func_OPHL(x,y,w,200,iprob);
  if iprint == 1
   fprintf('Lebesgue constant = %0.5f \n',maxL)
   return
  end
 end % if prax
 
 if iprint == 1
  fprintf(' iter = %d, nfunc = %d, exitflag = %d, f = %0.5f \n',iter,nf,exitflag,prax)
 end
 
 % convergence test
 if (abs(prax - prax_old) / prax_old) <= epsi / 3
  break
 else
  prax_old = prax;
 end % if abs
end % for k (improvement iterations)

if iprint == 1
 fprintf('\n total number of function evaluations = %d \n',nftot)
end

% maxL = prax;
x = xp(1:n)'; 
y = xp(n+1:n2)';

% this may not be enough to compute the L-constant reliably!!!!!!
[Psinp,maxL,Psidot,XY] = gm_comp_Lebesgue_func_OPHL(x,y,w,200,iprob);
if iprint == 1
 fprintf(' Lebesgue constant = %0.5f \n',maxL)
end

end % function

function f=const(x,y,X);

f = 1;

end


