function [x_opt,y_opt,maxL]=gm_print_points_dist_disk(degree,epsi,iprint,nfev,x0,y0);
%GM_PRINT_POINTS_DIST_DISK prints sets of points for the unit disk using distance maximization 

% it gives points which are regularly distributed
% they have a large Lebesgue constant!


% Input:
% degree = given degree
% epsi = stopping criteria on the value of the objective function
% iprint = 1 with printing
% nfev = approximate maximum number of function evaluations in praxis
% x0, y0 = starting points
%
% Output:
% x_opt, y_opt = coordinates of the set of points
% maxL  = approximate Lebesgue constant
%

%
% Author G. Meurant
% February 2017
%

global WAM distance

% distance is 'Dub' (Dubiner distance) or 'dist' (Euclidean distance)
distance = 'Dub';

if nargin < 3
 iprint = 0;
end

if nargin < 4
 nfev = 2000;
end

if degree > 20
 fprintf('\n gm_print_points_dist_disk: The degree may be too large, be patient! \n\n')
end
  
pb = 'disk';
WAM = gm_disk_wam(300);
  
if iprint == 1
 fprintf('\n Degree = %d, Problem %s \n\n',degree,pb)
end

% equal weights
n = ((degree + 1) * (degree + 2)) / 2;
w = ones(n,1) / n;

if nargin < 5
 
 start_points = 'rad';
 
 if strcmpi(start_points,'leja') == 1
  % Leja-like points for starting
  leja = gm_leja_disk(degree);
  x0 = leja(:,1);
  y0 = leja(:,2);
 elseif strcmpi(start_points,'rad') == 1
  % Optimal radii points
  [ptsrad,radius,maxL] = gm_points_radius_opt_disk(degree);
  x0 = ptsrad(:,1);
  y0 = ptsrad(:,2);
 else
  [ptsF,maxL] = gm_pointsF_rad_disk(degree);
  x0 = ptsF(:,1);
  y0 = ptsF(:,2);
 end
 
end % if nargin


% distance maximization

if iprint == 1
 fprintf('\n --------Start of distance maximization \n\n')
end

minimizer = 'praxis';

if strcmpi(minimizer,'praxis') == 1
 [xl,yl] = gm_min_praxis_distance_disk(x0,y0,epsi,iprint,nfev);
else
   % this solver is in the Matlab optimization toolbox
  X0 = [x0; y0];
  options = optimset('MaxIter',fix(nfev/10),...
   'MaxFunEvals',nfev,...
   'Display','iter-detailed',...
   'Diagnostics','off',...
   'TolX',epsi,...
   'TolFun',epsi,...
   'DiffMaxChange',0.1);
  
  A = []; B = []; Aeq = []; Beq = []; LB = []; UB = [];
  
  point = fminimax(@gm_dist_Dub_fcn_disk,X0,A,B,Aeq,Beq,LB,UB,[],options); 
  xl = point(1:n);
  yl = point(n+1:end);
  
end % if strcmpi

% [ptsD,maxL] = gm_points_dist_disk(degree);
% xl = ptsD(:,1);
% yl = ptsD(:,2);

gm_print_points(xl,yl);

x_opt = xl; y_opt = yl;

if strcmpi(distance,'dist') == 1
 [dmat,dmati,vd] = gm_dist_mat(x_opt,y_opt);
else
 [dmat,dmati,vd] = gm_dist_Dub_mat(x_opt,y_opt);
end

% minimal distances
mind = min(dmat+100*eye(n));

% recompute the Lebesgue contant on a fine mesh
pts = gm_disk_wam(500);
[maxL,Psidot] = gm_compXY_Lebesgue_func_OPHL(x_opt,y_opt,w,pts,2);

if iprint == 1
%  fprintf('\n min of distances = %g, max of distances = %g \n',min(min(dmat+100*eye(n))),max(max(dmat)))
 fprintf('\n smallest minimal distance = %g, largest minimal distance = %g \n',min(mind),max(mind))
 fprintf('\n --------End of the distance maximization, Lebesgue constant = %g \n',maxL)
 pause
 gm_clearallfig
end

% % refinement algorithm

it = 3; % number of iterations

if iprint == 1
 fprintf('\n --------Start of refinement, %d iterations \n',it)
end

% [xr,yr,Psinp,maxLr,x_opt,y_opt] = gm_Leb_refP_OPHL(xl,yl,w,xl,yl,maxLl,it,epsi,iprint);
[xr,yr,Psinp,maxLr,x_opt,y_opt] = gm_Leb_ref_disk(xl,yl,w,xl,yl,maxL,it,iprint);

% best points so far
% gm_print_points(x_opt,y_opt);

if iprint == 1
 fprintf('\n --------End of the refinement, Lebesgue constant after this step = %g \n',maxLr)
 pause
 gm_clearallfig
end

x_opt = xr; y_opt = yr;
% maxLr = maxLl;

[Psinp,maxL,Psidot,XY] = gm_comp_Lebesgue_func_OPHL(x_opt,y_opt,w,100,2);

% one point minimization

it = 5; % number of iterations

if iprint == 1
 fprintf('\n --------Start of one point minimization, %d iterations \n',it)
end

[x1,y1,Psinp,maxL1,x_opt,y_opt]=gm_Leb_one_disk_OPHL(x_opt,y_opt,w,x_opt,y_opt,maxLr,it,epsi,iprint);

% Leja order of the best points
ii = sqrt(-1);
z = gm_leja_ord(x_opt + ii * y_opt,n);
x_opt = real(z);
y_opt = imag(z);

% best points so far
gm_print_points(x_opt,y_opt);

if iprint == 1
 fprintf('\n --------End of the one point minimization, Lebesgue constant after this step = %g \n',maxL1)
end

% recompute the Lebesgue contant on a fine mesh
[maxL,Psidot] = gm_compXY_Lebesgue_func_OPHL(x_opt,y_opt,w,pts,2);

if iprint == 1
 fprintf('\n end of degree %d, Lebesgue constant = %g \n',degree,maxL)
 fprintf('\n -------------------------------------------------------------------- \n')
end







