function [Psinp,maxL,maxbnd,Psidot,XY]=gm_viz_Lebesgue_func(Hx,Hy,rowxy,V,Beta,x,y,w,npts,ipb);
%GM_VIZ_LEBESGUE_FUNC visualization of the Lebesgue function
% using the bivariate orthogonal polynomials

% Input:
% Hx, Hy, rowxy = arrays from gm_bivar_orthpol
% x, y = points defining the discrete inner product
% w = weights (sum(w) = 1)
% npts = number of vizulization points in one direction
% ipb = problem number
%        ipb = 1 square
%        ipb = 2 disk
%        ipb = 3 L-shape region
%        ipb = 4 triangle (simplex)
%        ipb = 5 double bubble
%        ipb = 6 ellipse
%        ipb = 7 half-ellipse
%
% Output:
% Psinp = values of the Lebesgue function
% maxL = Lebesgue constant
% maxbnd = max of a bound of the Lebesgue constant (or zero)
% Psidot = Psinp without reordering
% XY = coordinates of test points

%
% Author G. Meurant
% May 2014
% Updated July 2015
%

% communication with gm_indic_func
global iprob

% set the global variable
iprob = ipb;

Psinp = 0; 
maxL = 0; 
maxbnd = 0;
% test points, linear = 1, Cheb max = 2
testp = 2;

% set up the mesh on a rectangle including the domain

switch ipb
 
 case 1
  xmin = -1; ymin = -1;
  xmax = 1; ymax = 1;
  
 case 2
  xmin = -1;, ymin = -1;
  xmax = 1; ymax = 1;
  
 case 3
  xmin = -1;, ymin = -1;
  xmax = 1; ymax = 1;
  
 case 4
  xmin = 0;, ymin = 0;
  xmax = 1; ymax = 1;
  
 case 5
  xmin = -5; ymin = -5;
  xmax = 9; ymax = 5;
  
 case 6
  xmin = -2; ymin = -1;
  xmax = 2; ymax = 1;
  
 case 7
  % half-ellipse
  xmin = -2; ymin = 0;
  xmax = 2; ymax = 1;
  
 otherwise
  error('gm_viz_Lebesgue_func:: Wrong value of ipb')
end % switch

% eventually enlarge a bit the rectangle
fmin = 0.99;
fmax = 1.01;

rect = 1;
if rect == 1
 fmin = 1; fmax = 1;
end % if rect

if xmin > 0
 xmin = fmin * xmin;
else
 xmin = fmax * xmin;
end
if xmax > 0
 xmax = fmax * xmax;
else
 xmax = fmin * xmax;
end
if ymin > 0
 ymin = fmin * ymin;
else
 ymin = fmax * ymin;
end
if ymax > 0
 ymax = fmax * ymax;
else
 ymax = fmin * ymax;
end

% mesh in x and y
switch testp
 case 1
  % linear
  xx = linspace(xmin,xmax,npts);
  yy = linspace(ymin,ymax,npts);
  
 case 2
  % Chebyshev
  xx = gm_cheb_max(xmin,xmax,npts);
  yy = gm_cheb_max(ymin,ymax,npts);
  
 otherwise
  error('gm_viz_Lebesgue_func: testp must be 1 or 2')
end % switch

XY = [repmat(xx',npts,1) kron(yy,ones(1,npts))'];

% values at the inner product points using eigenvectors
[Phi,P] = gm_val_house(V,Beta,w);

% coefficients of the expansion
alp = Phi' * diag(w);

% compute the values of the polynomials
Psi = gm_eval_orthpol(XY,1,Hx,Hy,rowxy,size(Hx,1));

np = size(Psi,1);
n = size(Hx,1);

Psinp = zeros(np,1);
Pbnd = zeros(np,1);
for j = 1:n
 sk = zeros(np,1);
 
 % values of the j-th Lagrange polynomial
 for l = 1:np
  for k = 1:n
   sk(l) = sk(l) + alp(k,j) * Psi(l,k);
  end % for k
 end % for l
 
 Psinp = Psinp + abs(sk);
 
 Psidot = Psinp;
 
end % for j

for i = 1:np
 if gm_indic_func([XY(i,1); XY(i,2)]) == 0
  % value outside the domain
  Psinp(i) = 1;
 end % if
end % for i

maxL = max(Psinp);

% value of the bound???
sk = zeros(np,1);
for l = 1:np
 for j = 1:n
  sk(l) = sk(l) + abs(Psi(l,j));
 end % for j
end % for l
Pbnd = sk;

for i = 1:np
 if gm_indic_func([XY(i,1); XY(i,2)]) == 0
  % value outside the domain
  Pbnd(i) = 1;
 end % if
end % for i

zmin = min(Psinp);
zmax = max(Psinp);
if zmin > 0
 zmin = fmin * zmin;
else
 zmin = fmax * zmin;
end % if zmin
if zmax > 0
 zmax = fmax * zmax;
else
 zmax = fmin * zmax;
end % if zmax

% reshape the vector to conform to the mesh
Psinp = reshape(Psinp,npts,npts)';

Pbnd = reshape(Pbnd,npts,npts)';

%  surfl(xx,yy,Psinp);
shading interp
% change the color map?
%  colormap jet
%  colormap hsv
%  colormap prism
%  colormap gray
%  colormap hot
%  colormap summer
%  colormap spring
%  colormap bone

% more traditional surface
surf(xx,yy,Psinp)

title(['Lebesgue function, Leb const = ' num2str(maxL)])
axis([xmin xmax ymin ymax zmin zmax]);
xlabel('x');
ylabel('y');

return

% bound of the Lebesgue function
figure

surf(xx,yy,Pbnd)

zmin = min(min(Pbnd));
zmax = max(max(Pbnd));
if zmin > 0
 zmin = fmin * zmin;
else
 zmin = fmax * zmin;
end
if zmax > 0
 zmax = fmax * zmax;
else
 zmax = fmin * zmax;
end
maxbnd = zmax;

title('Bound')
axis([xmin xmax ymin ymax zmin zmax]);
xlabel('x');
ylabel('y');

figure

diff = Pbnd - Psinp;

surf(xx,yy,diff)

zmin = min(min(diff));
zmax = max(max(diff));
if zmin > 0
 zmin = fmin * zmin;
else
 zmin = fmax * zmin;
end
if zmax > 0
 zmax = fmax * zmax;
else
 zmax = fmin * zmax;
end

title('Difference')
axis([xmin xmax ymin ymax zmin zmax]);
xlabel('x');
ylabel('y');


