function [Psinp,maxL,Psidot,XY,Psidott]=gm_viz_Lebesgue_func_p_OPHL(x,y,w,npts,ipb,p);
%GM_VIZ_LEBESGUE_FUNC_P_OPHL visualization of the p-Lebesgue function
% for bivariate orthogonal polynomials using Huhtanen-Larsen

% This works only for some examples (because of the definition of the
% enclosing rectangle)

% p-norm

% Input:
% (x,y) = the points defining the discrete inner product
% w = weights, sum(w) = 1
% npts = number of vizualization points in one direction
% ipb = problem number
%        ipb = 1 square
%        ipb = 2 disk
%        ipb = 3 L-shape region
%        ipb = 4 triangle (simplex)
%        ipb = 5 double bubble
%        ipb = 6 ellipse
%        ipb = 7 half-ellipse
% p = integer defining the p-norm
%
% Output:
% Psinp = values of the Lebesgue function at test points
% maxL = approximate Lebesgue const
% Psidot = values of the Lebesgue function as a vector
% XY = test points

%
% Author G. Meurant
% May 2014
% Updated July 2015
%

% This is to communicate with gm_indic_func
global iprob

% set the global variable
iprob = ipb;

Psinp = 0; 
maxL = 0; 
maxbnd = 0;

% test points, linear = 1, Cheb max = 2
testp = 2;

% set up the mesh on a rectangle including the domain

switch ipb
 
 case 1
  xmin = -1; ymin = -1;
  xmax = 1; ymax = 1;
  
 case 2
  xmin = -1;, ymin = -1;
  xmax = 1; ymax = 1;
  
 case 3
  xmin = -1;, ymin = -1;
  xmax = 1; ymax = 1;
  
 case 4
  xmin = 0;, ymin = 0;
  xmax = 1; ymax = 1;
  
 case 5
  xmin = -5; ymin = -5;
  xmax = 9; ymax = 5;
  
 case 6
  xmin = -2; ymin = -1;
  xmax = 2; ymax = 1;
  
 case 7
  xmin = -2; ymin = 0;
  xmax = 2; ymax = 1;
  
 otherwise
  error('gm_viz_Lebesgue_func_OPHL: Wrong value of ipb')
end % switch

if nargin == 4
 xmin = min(x); xmax = max(x);
 ymin = min(y); ymax = max(y);
end

% eventually enlarge a bit the rectangle
fmin = 0.99;
fmax = 1.01;

rect = 1;
if rect == 1
 fmin = 1; fmax = 1;
end % if rect

if xmin > 0
 xmin = fmin * xmin;
else
 xmin = fmax * xmin;
end
if xmax > 0
 xmax = fmax * xmax;
else
 xmax = fmin * xmax;
end
if ymin > 0
 ymin = fmin * ymin;
else
 ymin = fmax * ymin;
end
if ymax > 0
 ymax = fmax * ymax;
else
 ymax = fmin * ymax;
end

% mesh in x and y
switch testp
 
 case 1
  % linear
  xx = linspace(xmin,xmax,npts);
  yy = linspace(ymin,ymax,npts);
  
 case 2
  % Chebyshev
  xx = gm_cheb_max(xmin,xmax,npts);
  yy = gm_cheb_max(ymin,ymax,npts);
  
 otherwise
  error('gm_viz_Lebesgue_func_OPHL: testp must be 1 or 2')
end % switch

% set up the mesh
XY = [repmat(xx',npts,1) kron(yy,ones(1,npts))'];

% total degree
n = length(x);
d = ceil((-3 + sqrt(1 + 8 * n)) / 2);

% values at the inner product points using Huhtanen-Larsen
[Phi,A,xy] = gm_OPHL(d,x,y,w);

% coefficients of the expansion
alp = Phi' * diag(w);

% compute the values of the polynomials
Psi = gm_OPHL_eval(d,XY(:,1),XY(:,2),w,A,1);

np = size(Psi,1);

Psinp = zeros(np,1);

for j = 1:n
 sk = zeros(np,1);
 
 % values of the j-th Lagrange polynomial
 for k = 1:n
  sk = sk + alp(k,j) * Psi(:,k);
 end % for k
 
 Psinp = Psinp + abs(sk).^p;
 
end % for j

Psinp = Psinp.^(1/p);

zz = realmax;
for i = 1:np
 if gm_indic_func([XY(i,1); XY(i,2)]) == 1
  zz = min(zz,Psinp(i));
 end % if
end % for i

for i = 1:np
 if gm_indic_func([XY(i,1); XY(i,2)]) == 0
  % value outside the domain
%   Psinp(i) = 1;
  Psinp(i) = zz;
 end % if
end % for i

zmin = min(Psinp);
zmax = max(Psinp);
if zmin > 0
 zmin = fmin * zmin;
else
 zmin = fmax * zmin;
end
if zmax > 0
 zmax = fmax * zmax;
else
 zmax = fmin * zmax;
end

maxL = zmax;

Psidot = Psinp;

if zmin == zmax
 temp = zmin;
 zmin = 0.9 * zmin;
 zmax = 1.1 * temp;
end

% if ipb == 2 && npts >= 100
%  XY = gm_disk_wam(npts);
%  [maxL,Psidott] = gm_compXY_Lebesgue_func_OPHL(x,y,w,XY,ipb);
% else
%  Psidott = Psidot;
% end
Psidott = Psidot;

% reshape the vector to conform to the mesh
Psinp = reshape(Psinp,npts,npts)';

%  surfl(xx,yy,Psinp);
shading interp
% change the color map?
%  colormap jet
%  colormap hsv
%  colormap prism
%  colormap gray
%  colormap hot
%  colormap summer
%  colormap spring
%  colormap bone

% more traditional surface
surf(xx,yy,Psinp)

title(['Lebesgue p-function, p = ' num2str(p) ', Leb const = ' num2str(maxL)])
axis([xmin xmax ymin ymax zmin zmax]);
xlabel('x');
ylabel('y');




